﻿##### Scriptinfo ##################################################################################
# Scriptreihe	    PAM-AdminGUI\ProxyFunctions
# Datum:            2020-02-29
# Version:          V1.10
# Programmierer:    Stephan Walther
###################################################################################################

#region lese AD-Infos und Variablen    
    Get-Variable -Scope global | Where-Object {'WS_OUAdminGruppen','WS_OUAdmins' -contains $_.name} | Remove-Variable -Scope global

    Get-Content -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.ini" | 
        ForEach-Object {
            if ($_.length -gt 0) {
                if ('[' -notcontains ($_)[0]) {
                    if ($_ -like '*==*') {                    
                        $Name,$Wert = ($_ -split '#')[0].Trim() -split '=='

                        $Name = "WS_" + $Name.trim()

                        Get-Variable -Scope global | Where-Object {$_.name -eq $Name} | Remove-Variable -Scope global
                        New-Variable -Name $Name.trim() -Value $Wert.trim() -Scope global
                    }
                    if ($_ -like '*+=*') {
                        $Name,$Wert = ($_ -split '#')[0].Trim() -split '\+='

                        $Name = "WS_" + $Name.trim()

                        if ((Get-Variable -Scope global | Where-Object {$_.name -eq $Name}) -EQ $null) {
                            New-Variable -Name $Name.trim() -Value @() -Scope global
                        }                        
                        $cur = @()
                        $cur += (Get-Variable -Name $Name.Trim()).value
                        $cur += $wert.trim() 

                        Set-Variable -Name $Name.trim() -Value $cur
                    } 
                }
            }
        }

    $Gruppen = @()
    if ($WS_MitDomainAdmins       -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-512'}).SamAccountName } 
    if ($WS_MitSchemaAdmins       -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-518'}).SamAccountName } 
    if ($WS_MitEnterpriseAdmins   -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-519'}).SamAccountName }
    if ($WS_MitProtectedUsers     -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Samaccountname -eq "Protected Users"'        ).SamAccountName }
    if ($WS_MitMXOrganizationMgmt -eq 'an') { $Gruppen += (Get-ADGroup -Filter 'Samaccountname -eq "Organization Management"').SamAccountName }

    $WS_OUAdminGruppen | ForEach-Object {
        $Gruppen += (Get-ADGroup -Filter ($WS_FilterAdminGruppen -replace '"') -SearchBase ($_ -replace '"')).SamAccountName    
    }
    $Gruppen    = "'" + ($Gruppen -join "','") + "'"
    
    $WS_OUAdmins | ForEach-Object {
        $Benutzer   = "'" + ((Get-ADUser -Filter ($WS_FilterAdmins -replace '"') -SearchBase ($_ -replace '"')).SamAccountName -join "','") + "'"
    }

    $DomainControllerAlle = "'" + ((get-addomaincontroller -Filter *).name -join "','") + "'"

    $Zeitspanne = $WS_AdminZeitInMinuten
#endregion
    
#region Funktion AD-Replication
$Function = @'
function repliziere-ADChanges {
    param(
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN% )] [string] $Gruppe, 
        [parameter(Mandatory=$true)] [ValidateSet( %DCs% )]     [string] $DCServer
    )

    Get-ADGroup -Identity $Gruppe | Sync-ADObject -Destination $DCServer
}
'@
$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%DCs%',$DomainControllerAlle
Invoke-Expression -Command $Function
#endregion

#region Function erstelle-PAMGruppenMitglied
$Function = @'
function erstelle-PAMGruppenMitglied {
    [cmdletbinding()]
    param(
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )]   [string] $Gruppe,
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )]   [string] $Admin,
        [parameter(Mandatory=$true)] [ValidateSet( %ZEITSPANNE% )] [string] $Zeitspanne
    )

    try {
        $Error.Clear()
        Add-ADGroupMember -Identity $Gruppe -Members $Admin -MemberTimeToLive (New-TimeSpan -Minutes $Zeitspanne) -ErrorAction SilentlyContinue
        Return 'OK'
    } 
    catch {
        Return ($error[0].Exception.Message)
    }
}
'@

$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%BENUTZER%',$Benutzer
$Function = $Function -replace '%ZEITSPANNE%',$Zeitspanne
Invoke-Expression -Command $Function
#endregion

#region Function entferne-PAMGruppenMitglied
$Function = @'
function entferne-PAMGruppenMitglied {
    [cmdletbinding()]
    param(
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )] [string] $Gruppe,
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )] [string] $Admin
    )

    try {
        $Error.Clear()
        $AdminDN = (Get-ADUser -Identity $Admin).DistinguishedName

        $Member = (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members | Where-Object { $_ -like "*$AdminDN" }

        $Member = $Member -split '>,'
        if ($Member.Count -eq 2) {
            Remove-ADGroupMember -Identity $Gruppe -Members $Member[1] -Confirm:$false
            Return "OK"
        } else {
            Return "Statische Gruppenmitglieder können nicht entfernt werden!" 
        }
    } 
    catch {
        Return ($error[0].Exception.Message)
    }
}
'@

$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function
#endregion

#region Function liste-Admins
$Function = @"
function liste-Admins {
    Return ($Benutzer -split ',' | Sort-Object)
}
"@
Invoke-Expression -Command $Function
#endregion

#region Function liste-Gruppen
$Function = @"
function liste-Gruppen {
    Return ($Gruppen -split ',' | Sort-Object)
}
"@
Invoke-Expression -Command $Function
#endregion

#region Function liste-Zeitspannen
$Function = @"
function liste-Zeitspannen {
    Return ([int[]] ($Zeitspanne -split ',') | Sort-Object)
}
"@
Invoke-Expression -Command $Function
#endregion

#region Function liste-DomainController
$Function = @'
function liste-DomainController {
    (get-addomaincontroller -Filter *).name | 
        Where-Object { $_ -notcontains ($env:computername) } |
            sort-object
}
'@
Invoke-Expression -Command $Function
#endregion

#region Function zeige-GruppenMitglieder
$Function = @'
function zeige-GruppenMitglieder {
    param (
        [parameter(Mandatory=$true)] [ValidateSet( %GRUPPEN%  )] [string] $Gruppe
    )

    (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members | 
        ForEach-Object {   
            $User = $_ -split '>,'
            if ($User.count -gt 1) {$User = $User[1]} else {$User = $User[0]}
            
            if ((Get-ADObject $User).ObjectClass -eq 'user') { 
                #if ($User.Count -gt 1) {$USer = $User[1]} else {$USer = $User[0]}           

                $SAM = (Get-ADUser -Identity $User).SamAccountName
                $TTL  = (($_ -split 'CN=')[0] -split '=')[-1] -replace '>' -replace ','
                $bis  = (($_ -split 'CN=')[0] -split '=')[-1] -replace '>'
                if ($bis) {                    
                    $bis = get-date -date (Get-Date).AddSeconds($bis) -Format "yyyy-MM-dd HH:mm:ss"
                }

                $out = New-Object -TypeName psobject
                $out | Add-Member -MemberType NoteProperty -Name 'User' -Value $SAM
                $out | Add-Member -MemberType NoteProperty -Name 'TTL'  -Value $TTL
                $out | Add-Member -MemberType NoteProperty -Name 'bis'  -Value $bis
                $out
            } 
        }
}
'@
$Function = $Function -replace '%GRUPPEN%',$Gruppen
Invoke-Expression -Command $Function
#endregion

#region Function zeige-Mitgliedschaften
$function = @'
function Get-RecursiveUserGroupMembership {
    [CmdletBinding()]
    param(        
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )] [String] $UserName
    )

    begin {
        $ADGroupCache = @{}
        $UserGroups = @{}
        $results = @()
        
        function __findPath ([string]$currentGroup) {            
            if (!$UserGroups.ContainsKey($currentGroup)) {
                $groupObject = if ($ADGroupCache.ContainsKey($currentGroup)) {
                    Write-Verbose "Found group in cache: $currentGroup"
                    $ADGroupCache[$currentGroup]
                } else {
                    Write-Verbose "Group: $currentGroup is not presented in cache. Retrieve and cache."
                    $g = Get-ADGroup -Identity $currentGroup -Property "MemberOf"
                    
                    $ADGroupCache.Add($g.DistinguishedName, $g)
                    $g
                }
                
                $UserGroups.Add($currentGroup, $groupObject)
                Write-Verbose "Member of: $currentGroup"
                foreach ($p in $groupObject.MemberOf) {
                    __findPath $p
                }
            } else {
                Write-Verbose "Closed walk or duplicate on '$currentGroup'. Skipping."
            }
        }        
    }

    process {
        foreach ($user in $UserName) {
            Write-Verbose "========== $user =========="
            
            $UserObject = Get-ADUser -Identity $user -Property "MemberOf"
            $UserObject.MemberOf | ForEach-Object { __findPath $_ }

            $results += New-Object psobject -Property @{
                UserName = $UserObject.Name;
                MemberOf = $UserGroups.Values | % {$_}; 
            }
            $UserGroups.Clear()
        }
    }

    end {
        ($results.memberof | Sort-Object -Property samaccountname).samaccountname
    }
}
'@
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function

$Function = @'
function zeige-Mitgliedschaften {
    param (
        [parameter(Mandatory=$true)] [ValidateSet( %BENUTZER% )] [string] $Admin
    )
    
    $AdminGruppen = ($Gruppen -replace "'") -split ','
    $AdminDN = (Get-ADUser -Identity $Admin).DistinguishedName
    $IsProtected = $false 

    (Get-ADUser -Identity $Admin -Properties memberof).memberof | 
        ForEach-Object {
            $Gruppe = (Get-ADGroup -Identity $_).SamAccountName        
            $Member = (Get-ADGroup -Identity $Gruppe -ShowMemberTimeToLive -Properties members).members | Where-Object { $_ -like "*$AdminDN" }
            
            if ($AdminGruppen -contains $Gruppe) {
                $TTL    = (($Member -split 'CN=')[0] -split '=')[-1] -replace '>'            
                if ($TTL) {
                    $bis = get-date -date (Get-Date).AddSeconds($TTL) -Format "yyyy-MM-dd HH:mm:ss"
                } else {
                    $bis = ''
                }

                $out = New-Object -TypeName psobject
                $out | Add-Member -MemberType NoteProperty -Name 'Gruppe' -Value $Gruppe
                $out | Add-Member -MemberType NoteProperty -Name 'TTL'    -Value $TTL
                $out | Add-Member -MemberType NoteProperty -Name 'bis'    -Value $bis
                $out

                if ($Gruppe -eq "Protected Users") {$IsProtected = $true}
            }        
        }

    if ($WS_MitProtectedUsers -eq 'an' -and $IsProtected -eq $false) {
        if (Get-RecursiveUserGroupMembership -UserName $Admin | Where-Object { $_ -eq "Protected Users" }) {
            $out = New-Object -TypeName psobject
            $out | Add-Member -MemberType NoteProperty -Name 'Gruppe' -Value "Protected Users (indirect)"
            $out | Add-Member -MemberType NoteProperty -Name 'TTL'    -Value ""
            $out | Add-Member -MemberType NoteProperty -Name 'bis'    -Value ""
            $out
        }
    }
}
'@
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function
#endregion

#region Function entferne-Mitgliedschaften
$Function = @'
function entferne-Mitgliedschaften {
    param (
        [parameter(Mandatory=$true,ParameterSetName="Benutzer")] [ValidateSet( %BENUTZER% )] [string] $Admin,
        [parameter(Mandatory=$true,ParameterSetName="Gruppe"  )] [ValidateSet( %GRUPPEN%  )] [string] $Gruppe
    )
    
    if ($Admin) {
        $AdminGruppen = ($Gruppen -replace "'") -split ','
        $AdminDN = (Get-ADUser -Identity $Admin).DistinguishedName
        $StaticMember = $false

        try {
            (Get-ADUser -Identity $Admin -Properties memberof).memberof | 
                ForEach-Object {
                    $CurGruppe = (Get-ADGroup -Identity $_).SamAccountName        
                    $CurMember = (Get-ADGroup -Identity $CurGruppe -ShowMemberTimeToLive -Properties members).members | Where-Object { $_ -like "*$AdminDN" }
            
                    if ($AdminGruppen -contains $CurGruppe) {
                        $TTL    = (($CurMember -split 'CN=')[0] -split '=')[-1] -replace '>'            
                        if ($TTL) {
                            Remove-ADGroupMember -Identity $CurGruppe -Members $Admin -Confirm:$false
                        } 
                    } else {
                        $StaticMember = $true
                    }
                }
            if ( $StaticMember ) { "Hinweis##Statische Mitgliedschaften bleiben bestehen." } else { "OK" }
        }
        catch {
            "FEHLER##$($error[0].exception.message)"
        }
    } else {        
        try {
            (Get-ADGroup -Identity $Gruppe -Properties members -ShowMemberTimeToLive).members |
                Where-Object { $_ -like '<TTL=*' } |
                    ForEach-Object {
                        $Member = ($_ -split '>,')[-1]

                        Remove-ADGroupMember -Identity $Gruppe -Members $Member -Confirm:$false
                    }
            "OK"
        }
        catch {
            "FEHLER##$($error[0].exception.message)"
        }
    }
}
'@
$Function = $Function -replace '%GRUPPEN%',$Gruppen
$Function = $Function -replace '%BENUTZER%',$Benutzer
Invoke-Expression -Command $Function
#endregion