﻿##### Scriptinfo ##################################################################################
# Scriptreihe	    PAM-AdminGUI\Setup-PAM&JEA
# Datum:            2020-02-29
# Version:          V1.10
# Programmierer:    Stephan Walther
###################################################################################################

cd 'M:\Zwischenablage'

#region Variablen (immer ausführen)
    notepad.exe PAM-AdminGUI.ini
    pause
 
    Get-Variable -Scope global | Where-Object {'WS_OUAdminGruppen','WS_OUAdmins' -contains $_.name} | Remove-Variable -Scope global

    Get-Content -Path "PAM-AdminGUI.ini" | 
        ForEach-Object {
            if ($_.length -gt 0) {
                if ('[' -notcontains ($_)[0]) {
                    if ($_ -like '*==*') {                    
                        $Name,$Wert = ($_ -split '#')[0].Trim() -split '=='

                        $Name = "WS_" + $Name.trim()

                        Get-Variable -Scope global | Where-Object {$_.name -eq $Name} | Remove-Variable -Scope global
                        New-Variable -Name $Name.trim() -Value $Wert.trim() -Scope global
                    }
                    if ($_ -like '*+=*') {
                        $Name,$Wert = ($_ -split '#')[0].Trim() -split '\+='

                        $Name = "WS_" + $Name.trim()

                        if ((Get-Variable -Scope global | Where-Object {$_.name -eq $Name}) -EQ $null) {
                            New-Variable -Name $Name.trim() -Value @() -Scope global
                        }                        
                        $cur = @()
                        $cur += (Get-Variable -Name $Name.Trim()).value
                        $cur += $wert.trim() 

                        Set-Variable -Name $Name.trim() -Value $cur
                    } 
                }
            }
        }
#endregion
        
#region Setup Privileged Access Management - PAM (nur auf dem ersten Server ausführen)
    # Vorbereitung PAM  
        Import-Module -Name ActiveDirectory 
        $ADFunktionsEbene = (Get-ADForest).ForestMode
        if ($ADFunktionsEbene -replace '\D' -lt 2016) {
            $Antwort = Read-Host -Prompt "Die Funktionsebene des AD ist $ADFunktionsEbene, es wird aber mind. Windows2016Forest benoetigt. Soll die Funktionsebene heraufgestuft werden? Der Vorgang ist nicht umkehrbar!!! (j|n)"
            if ($Antwort -eq 'j') {
                Set-ADForestMode -ForestMode "Windows2016Forest" -Identity (Get-ADForest).name
            } else {
                exit
            }         
        }
        if ((Get-ADOptionalFeature -Identity 'Privileged Access Management Feature').EnabledScopes.count -eq 0) {
            $Antwort = Read-Host -Prompt "Das Privileged Access Management Feature ist nicht aktiviert. Soll ich es aktivieren? Der Vorgang ist nicht umkehrbar!!! (j|n)"
            if ($Antwort -eq 'j') {
                Enable-ADOptionalFeature -Identity 'Privileged Access Management Feature' -Target (Get-ADForest).name -Scope ForestOrConfigurationSet
                Wait-Event -Timeout 5
            } else {
                exit   
            }
        }

        if ((Get-ADOptionalFeature -Identity 'Privileged Access Management Feature').EnabledScopes.count -eq 0) {
            Write-Host "PAM ist nicht aktiviert!" -ForegroundColor Yellow
            exit
        }

    # erstelle die AdminRollenGruppe
        New-ADGroup `
            -Name "$WS_PAMAdminRolleGruppeName" `
            -GroupCategory Security `
            -GroupScope DomainLocal `
            -Path $WS_PAMAdminRolleGruppeOU `
            -Description "Recht für JEA/PAM-AdminGUI"
#endregion

#region Setup/Update Just Enough Administration - JEA (lokal je JEA-Server ausführen)
    # erstelle PS-Module für die neue Rolle
        New-Item           -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI" -ItemType Directory -ErrorAction SilentlyContinue |Out-Null
        New-ModuleManifest -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.psd1"
        New-Item           -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\RoleCapabilities" -ItemType Directory -ErrorAction SilentlyContinue | Out-Null

    # Definition der Administrations-Rolle
        $Rolle = @{
            Author = $env:username
            CompanyName = (Get-ADDomain).NetBIOSName
            ModulesToImport = 'ActiveDirectory'
            VisibleCmdlets = 'add-member'
            ScriptsToProcess = "C:\Program Files\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI-Functions.ps1"
            FunctionDefinitions = @{ Name = 'Get-UserInfo'; ScriptBlock = { $PSSenderInfo } }
        }

    # erzeuge die Rollenbeschreibungsdatei
        New-PSRoleCapabilityFile -Path "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\RoleCapabilities\PAM-AdminGUI.psrc" @Rolle

    # kopiere FunctionsFile
        Copy-Item PAM-AdminGUI-Functions.ps1 -Destination "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI-Functions.ps1"
        Copy-Item PAM-AdminGUI.ini           -Destination "$env:ProgramFiles\WindowsPowerShell\Modules\PAM-AdminGUI\PAM-AdminGUI.ini"

    # erstelle Endpunkt-Sessionconfiguration
        $JEAConfigParameter = @{
            SessionType = 'RestrictedRemoteServer'
            RunAsVirtualAccount = $true
            RoleDefinitions = @{ "$((Get-ADDomain).NetBIOSName)\$WS_PAMAdminRolleGruppeName" = @{ RoleCapabilities = "PAM-AdminGUI"  } }
            TranscriptDirectory = "$env:ProgramData\JEAConfiguration\Transcripts"
        }
    
    # erstelle Verzeichnis für die Session-Konfigurationen
        New-Item -Path "$env:ProgramData\JEAConfiguration" -ItemType Directory -ErrorAction SilentlyContinue | Out-Null

    # erstelle JEA-Endpunkt
        if (Get-PSSessionConfiguration -Name 'PAM-AdminGUI' -ErrorAction SilentlyContinue) {
            Unregister-PSSessionConfiguration -Name 'PAM-AdminGUI' -ErrorAction Stop
        }
        New-PSSessionConfigurationFile -Path "$env:ProgramData\JEAConfiguration\PAM-AdminGUI.pssc" @JEAConfigParameter

    # lade die JEA-Session-Konfiguration
        Register-PSSessionConfiguration -Name 'PAM-AdminGUI' -Path "$env:ProgramData\JEAConfiguration\PAM-AdminGUI.pssc" -WarningAction SilentlyContinue

    # aktiviere ModuleLogging (optional)
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell -ErrorAction SilentlyContinue | out-null
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging -ErrorAction SilentlyContinue | out-null
        New-Item -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging\ModuleNames -ErrorAction SilentlyContinue | out-null
        Set-ItemProperty -Path HKLM:\SOFTWARE\Policies\Microsoft\Windows\Powershell\ModuleLogging\ModuleNames -Name '*' -Value '*' | Out-Null

#endregion